from django.db import migrations, models
import django.db.models.deletion
import huscy.project_ethics.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('projects', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Ethic',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(blank=True, default='', max_length=255, verbose_name='Code')),
            ],
            options={
                'verbose_name': 'Ethic',
                'verbose_name_plural': 'Ethics',
                'ordering': ('-project', 'ethic_board__name'),
            },
        ),
        migrations.CreateModel(
            name='EthicBoard',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'Ethic board',
                'verbose_name_plural': 'Ethic boards',
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='EthicFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('filetype', models.PositiveSmallIntegerField(choices=[(0, 'Ethic proposal'), (1, 'Ethic votum'), (2, 'Ethic amendment')], verbose_name='File type')),
                ('filehandle', models.FileField(upload_to=huscy.project_ethics.models.EthicFile.get_upload_path, verbose_name='File handle')),
                ('filename', models.CharField(max_length=255, verbose_name='File name')),
                ('uploaded_at', models.DateTimeField(auto_now_add=True, verbose_name='Uploaded at')),
                ('uploaded_by', models.CharField(editable=False, max_length=126, verbose_name='Uploaded by')),
                ('ethic', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='ethic_files', to='project_ethics.ethic', verbose_name='Ethic')),
            ],
            options={
                'verbose_name': 'Ethic file',
                'verbose_name_plural': 'Ethic files',
                'ordering': ('-ethic__project', '-ethic', 'filename'),
            },
        ),
        migrations.AddField(
            model_name='ethic',
            name='ethic_board',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='project_ethics.ethicboard', verbose_name='Ethic board'),
        ),
        migrations.AddField(
            model_name='ethic',
            name='project',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ethics', to='projects.project', verbose_name='Project'),
        ),
        migrations.AlterUniqueTogether(
            name='ethic',
            unique_together={('project', 'ethic_board')},
        ),
    ]



from django.contrib import admin
from reversion.admin import VersionAdmin

from huscy.project_ethics import models


@admin.register(models.EthicBoard)
class EthicBoardAdmin(VersionAdmin, admin.ModelAdmin):
    pass


@admin.register(models.Ethic)
class EthicAdmin(VersionAdmin, admin.ModelAdmin):
    list_display = 'id', 'project_title', 'ethic_board', 'code'
    search_fields = 'code', 'project__title'

    def project_title(self, ethic):
        return ethic.project.title


@admin.register(models.EthicFile)
class EthicFileAdmin(VersionAdmin, admin.ModelAdmin):
    date_hierarchy = "uploaded_at"
    fields = 'filetype', 'filehandle', 'filename'
    list_display = ('id', 'project_title', 'ethic_board', 'ethic_code', 'filetype', 'filename',
                    'uploaded_at', 'uploaded_by')
    list_display_links = 'id', 'filetype', 'filename'
    readonly_fields = 'filehandle',
    search_fields = 'ethic__project__title', 'ethic__code', 'filename', 'uploaded_by'

    def has_add_permission(self, request, ethic_file=None):
        return False

    def ethic_board(self, ethic_file):
        return ethic_file.ethic.ethic_board.name

    def ethic_code(self, ethic_file):
        return ethic_file.ethic.code

    def project_title(self, ethic_file):
        return ethic_file.ethic.project.title



import django.core.validators
from django.db import migrations, models
import huscy.project_ethics.models


class Migration(migrations.Migration):

    dependencies = [
        ('project_ethics', '0002_auto_20211101_0600'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ethicfile',
            name='filehandle',
            field=models.FileField(max_length=255, upload_to=huscy.project_ethics.models.EthicFile.get_upload_path, validators=[django.core.validators.FileExtensionValidator(allowed_extensions=['pdf'])], verbose_name='File handle'),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('project_ethics', '0003_alter_ethicfile_filehandle'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ethicfile',
            name='filetype',
            field=models.PositiveSmallIntegerField(choices=[(0, 'Proposal'), (1, 'Attachment'), (2, 'Vote'), (3, 'Cover letter')], verbose_name='File type'),
        ),
    ]



from django.urls import include, path
from rest_framework.routers import DefaultRouter
from rest_framework_nested.routers import NestedDefaultRouter

from huscy.project_ethics import views
from huscy.projects.urls import project_router


router = DefaultRouter()
router.register('ethicboards', views.EthicBoardViewSet)

project_router.register('ethics', views.EthicViewSet, basename='ethic')

ethic_router = NestedDefaultRouter(project_router, 'ethics', lookup='ethic')
ethic_router.register('ethicfiles', views.EthicsFileViewSet, basename='ethicfile')


urlpatterns = [
    path('api/', include(router.urls + project_router.urls + ethic_router.urls)),
]



from django.shortcuts import get_object_or_404
from rest_framework import mixins, viewsets
from rest_framework.permissions import DjangoModelPermissions, IsAuthenticated
from reversion import set_comment
from reversion.views import RevisionMixin

from huscy.projects.models import Project
from huscy.project_ethics import serializer
from huscy.project_ethics.models import Ethic
from huscy.project_ethics.permissions import EthicsFilePermission, EthicPermission
from huscy.project_ethics.services import get_ethic_boards, get_ethic_files, get_ethics


class EthicBoardViewSet(RevisionMixin, mixins.CreateModelMixin, mixins.DestroyModelMixin,
                        mixins.ListModelMixin, mixins.UpdateModelMixin, viewsets.GenericViewSet):
    http_method_names = 'delete', 'head', 'get', 'options', 'post', 'put'
    permission_classes = (DjangoModelPermissions, )
    queryset = get_ethic_boards()
    serializer_class = serializer.EthicBoardSerializer

    def perform_create(self, serializer):
        ethic_board = serializer.save()
        set_comment(f'Created ethic board "{ethic_board.name}"')

    def perform_destroy(self, ethic_board):
        ethic_board.delete()
        set_comment(f'Deleted ethic board "{ethic_board.name}"')

    def perform_update(self, serializer):
        ethic_board = serializer.save()
        set_comment(f'Updated ethic board "{ethic_board.name}"')


class EthicViewSet(RevisionMixin, mixins.CreateModelMixin, mixins.DestroyModelMixin,
                   mixins.ListModelMixin, mixins.UpdateModelMixin, viewsets.GenericViewSet):
    http_method_names = 'delete', 'head', 'get', 'options', 'post', 'put'
    permission_classes = IsAuthenticated, EthicPermission
    serializer_class = serializer.EthicSerializer

    def initial(self, request, *args, **kwargs):
        self.project = get_object_or_404(Project, pk=self.kwargs['project_pk'])
        super().initial(request, *args, **kwargs)

    def get_queryset(self):
        return get_ethics(self.project)

    def perform_create(self, serializer):
        ethic = serializer.save(project=self.project)
        set_comment(f'Created ethic <ID-{ethic.id}>')

    def perform_destroy(self, ethic):
        ethic.delete()
        set_comment(f'Deleted ethic <ID-{ethic.id}>')

    def perform_update(self, serializer):
        ethic = serializer.save()
        set_comment(f'Updated ethic <ID-{ethic.id}>')


class EthicsFileViewSet(RevisionMixin, mixins.CreateModelMixin, mixins.DestroyModelMixin,
                        mixins.UpdateModelMixin, viewsets.GenericViewSet):
    http_method_names = 'delete', 'head', 'options', 'post', 'put'
    permission_classes = IsAuthenticated, EthicsFilePermission

    def initial(self, request, *args, **kwargs):
        self.ethics = get_object_or_404(Ethic.objects.select_related('project'),
                                        pk=self.kwargs['ethic_pk'],
                                        project=self.kwargs['project_pk'])
        super().initial(request, *args, **kwargs)

    def get_queryset(self):
        return get_ethic_files(self.ethics)

    def get_serializer_class(self):
        if self.request.method == 'PUT':
            return serializer.UpdateEthicsFileSerializer
        return serializer.EthicsFileSerializer

    def perform_create(self, serializer):
        ethics_file = serializer.save(ethic=self.ethics, creator=self.request.user)
        set_comment(f'Created ethics file <ID-{ethics_file.id}>')

    def perform_destroy(self, ethics_file):
        ethics_file.delete()
        set_comment(f'Deleted ethics file <ID-{ethics_file.id}>')

    def perform_update(self, serializer):
        ethics_file = serializer.save()
        set_comment(f'Updated ethics file <ID-{ethics_file.id}>')



from rest_framework import serializers

from huscy.project_ethics import models, services


class EthicBoardSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.EthicBoard
        fields = 'id', 'name'

    def create(self, validated_data):
        return services.create_ethic_board(**validated_data)

    def update(self, ethic_board, validated_data):
        return services.update_ethic_board(ethic_board, **validated_data)


class EthicsFileSerializer(serializers.ModelSerializer):
    filename = serializers.CharField(required=False, default='')
    filetype_name = serializers.CharField(source='get_filetype_display', read_only=True)

    class Meta:
        model = models.EthicFile
        fields = (
            'id',
            'ethic',
            'filehandle',
            'filename',
            'filetype',
            'filetype_name',
            'uploaded_at',
            'uploaded_by',
        )

    def create(self, validated_data):
        return services.create_ethic_file(**validated_data)


class UpdateEthicsFileSerializer(serializers.ModelSerializer):
    filetype_name = serializers.CharField(source='get_filetype_display', read_only=True)

    class Meta:
        model = models.EthicFile
        fields = (
            'id',
            'ethic',
            'filehandle',
            'filename',
            'filetype',
            'filetype_name',
            'uploaded_at',
            'uploaded_by',
        )
        read_only_fields = 'filehandle',

    def update(self, ethics_file, validated_data):
        return services.update_ethic_file(ethics_file, **validated_data)


class EthicSerializer(serializers.ModelSerializer):
    ethic_board_name = serializers.CharField(source='ethic_board.name', read_only=True)
    ethic_files = EthicsFileSerializer(many=True, read_only=True)

    class Meta:
        model = models.Ethic
        fields = 'code', 'ethic_board', 'ethic_board_name', 'ethic_files', 'id', 'project'
        read_only_fields = 'project',

    def create(self, validated_data):
        return services.create_ethic(**validated_data)

    def update(self, ethic, validated_data):
        return services.update_ethic(ethic, **validated_data)



import django.core.validators
from django.db import migrations, models
import huscy.project_ethics.models


class Migration(migrations.Migration):

    dependencies = [
        ('project_ethics', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ethic',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='ethicboard',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='ethicfile',
            name='filehandle',
            field=models.FileField(upload_to=huscy.project_ethics.models.EthicFile.get_upload_path, validators=[django.core.validators.FileExtensionValidator(allowed_extensions=['pdf'])], verbose_name='File handle'),
        ),
        migrations.AlterField(
            model_name='ethicfile',
            name='filetype',
            field=models.PositiveSmallIntegerField(choices=[(0, 'Proposal'), (1, 'Attachment'), (2, 'Vote')], verbose_name='File type'),
        ),
        migrations.AlterField(
            model_name='ethicfile',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



from huscy.project_ethics.models import Ethic, EthicBoard, EthicFile


def create_ethic_board(name):
    return EthicBoard.objects.create(name=name)


def get_ethic_boards():
    return EthicBoard.objects.all()


def update_ethic_board(ethic_board, name):
    ethic_board.name = name
    ethic_board.save()
    return ethic_board


def create_ethic(project, ethic_board, code=''):
    return Ethic.objects.create(project=project, ethic_board=ethic_board, code=code)


def get_ethics(project):
    return Ethic.objects.filter(project=project)


def update_ethic(ethic, ethic_board, code):
    ethic.code = code
    ethic.ethic_board = ethic_board
    ethic.save()
    return ethic


def create_ethic_file(ethic, filehandle, filetype, creator, filename=''):
    filename = filename or filehandle.name.split('/')[-1]

    return EthicFile.objects.create(
        ethic=ethic,
        filehandle=filehandle,
        filename=filename,
        filetype=filetype,
        uploaded_by=creator.get_full_name(),
    )


def get_ethic_files(ethic):
    return EthicFile.objects.filter(ethic=ethic)


def update_ethic_file(ethic_file, filetype, filename):
    ethic_file.filetype = filetype
    ethic_file.filename = filename
    ethic_file.save()
    return ethic_file



from django.core.validators import FileExtensionValidator
from django.db import models
from django.utils.translation import gettext_lazy as _

from huscy.projects.models import Project


class EthicBoard(models.Model):
    name = models.CharField(_('Name'), max_length=255, unique=True)

    def __str__(self):
        return self.name

    class Meta:
        ordering = 'name',
        verbose_name = _('Ethic board')
        verbose_name_plural = _('Ethic boards')


class Ethic(models.Model):
    project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name='ethics',
                                verbose_name=_('Project'))
    ethic_board = models.ForeignKey(EthicBoard, on_delete=models.PROTECT,
                                    verbose_name=_('Ethic board'))
    code = models.CharField(_('Code'), max_length=255, blank=True, default='')

    class Meta:
        ordering = '-project', 'ethic_board__name'
        unique_together = 'project', 'ethic_board'
        verbose_name = _('Ethic')
        verbose_name_plural = _('Ethics')


class EthicFile(models.Model):
    class TYPE(models.IntegerChoices):
        proposal = 0, _('Proposal')
        votum = 1, _('Attachment')
        amendment = 2, _('Vote')
        cover_letter = 3, _('Cover letter')

    def get_upload_path(self, filename):
        project = self.ethic.project
        return f'projects/{project.id}/ethics/{filename}'

    ethic = models.ForeignKey(Ethic, on_delete=models.CASCADE, related_name='ethic_files',
                              editable=False, verbose_name=_('Ethic'))

    filetype = models.PositiveSmallIntegerField(_('File type'), choices=TYPE.choices)

    filehandle = models.FileField(_('File handle'), upload_to=get_upload_path, max_length=255,
                                  validators=[FileExtensionValidator(allowed_extensions=['pdf'])])
    filename = models.CharField(_('File name'), max_length=255)

    uploaded_at = models.DateTimeField(_('Uploaded at'), auto_now_add=True, editable=False)
    uploaded_by = models.CharField(_('Uploaded by'), max_length=126, editable=False)

    class Meta:
        ordering = '-ethic__project', '-ethic', 'filename'
        verbose_name = _('Ethic file')
        verbose_name_plural = _('Ethic files')



from rest_framework.permissions import BasePermission


class EthicPermission(BasePermission):

    def has_permission(self, request, view):
        if request.method == 'POST':
            return (request.user.has_perm('projects.change_project') or
                    request.user.has_perm('projects.change_project', view.project))

        return True

    def has_object_permission(self, request, view, obj):
        return (request.user.has_perm('projects.change_project') or
                request.user.has_perm('projects.change_project', view.project))


class EthicsFilePermission(BasePermission):

    def has_permission(self, request, view):
        if request.method == 'POST':
            return (request.user.has_perm('projects.change_project') or
                    request.user.has_perm('projects.change_project', view.ethics.project))

        return True

    def has_object_permission(self, request, view, obj):
        can_change_project = (
            request.user.has_perm('projects.change_project') or
            request.user.has_perm('projects.change_project', view.ethics.project)
        )

        if not can_change_project:
            return False

        '''
        if request.method == 'DELETE':
            return request.user.has_perm('project_ethics.delete_ethicfile')
        '''

        return True
